define('feature/repository/branch-selector-field-internals', [
    'jquery',
    'eve',
    'model/revision-reference',
    'widget/branch-selector-dialog'
], function (
    $,
    eve,
    RevisionReference,
    BranchSelectorDialog
) {

    /**
     * Provides an ID to pass to BranchSelectorDialog.
     * If a baseId is given, the id is prefixed with it, otherwise a unique ID is generated for each call.
     * The goal is to avoid unwanted ID clashes.
     *
     * @param baseId An ID prefix to use when generating a BranchSelectorDialog id.
     */
    var generateDialogId = (function() {
        // fallback global ID used to ensure that each branch selector dialog is unique
        // If the Soy template is used, we should always be provided with a baseId and globalId shouldn't actually
        // be necessary.
        var globalId = 1;
        return function(baseId) {
            return (baseId || globalId++) + '_branch_selector_field_dialog';
        };
    })();


    /**
     * Create a BranchSelectorDialog and link it with the given dialog $trigger and $removeSelection button.
     * @param $trigger an element that when clicked, should open the dialog;
     * @param $removeSelection an element that when clicked, should remove the current selection for the branch selector.
     * @return {BranchSelectorDialog}
     */
    function createDialog($trigger, $removeSelection) {
        var $input = $trigger.prev('input');
        var inputId = $input.attr('id');
        var dialogId = generateDialogId(inputId);

        // HACK: It's possible that an InlineDialog is left around from a previous invocation. This cleans it up.
        // To avoid this, BranchSelectorDialog would need to completely remove its InlineDialog, rather than show and hide it.
        $('#inline-dialog-' + dialogId).remove();

        var dialog = new BranchSelectorDialog($trigger, dialogId, {
            branchesOnly : !$input.data("show-tags"),
            disableLinks: !$input.data("enable-branch-links"),
            context: dialogId
        });
        eve.on('stash.widget.branchselector.revisionRefChanged', function(revisionRef, context) {
            if (context === dialogId) {
                $input.val(revisionRef ? revisionRef.getId() : '');
                dialog.updateSelectedRevisionReference(revisionRef);
                $removeSelection.toggleClass('hidden', !revisionRef);
            }
        });
        $trigger.data('dialog', dialog);
        return dialog;
    }

    // live-bind the clickable parts and setup a BranchSelectorDialog, if necessary.
    $(document).on('click', '.branch-selector-field', function(e) {
        var $trigger = $(this);
        var $removeSelection = $trigger.nextAll('.remove-link');
        if (!$trigger.data('dialog')) {
            e.preventDefault();
            createDialog($trigger, $removeSelection);
            $trigger.click();
        }
    }).on('click', '.branch-selector-field + .remove-link', function(e) {
        e.preventDefault();

        var $removeSelection = $(this);
        var $trigger = $removeSelection.prevAll('.branch-selector-field');
        var $input = $trigger.prevAll('input');

        var dialog = $trigger.data('dialog') || createDialog($trigger, $removeSelection);

        $input.val('');
        dialog.clearSelection();
        $removeSelection.addClass('hidden');
    });
    eve.on('stash.widget.branchselector.inputAdded', function(id) {
        var $input = $('#' + id);
        var refId = $input.val();

        if (!refId) {
            return;
        }

        var dialog = createDialog($input.nextAll('.branch-selector-field'), $input.nextAll('.remove-link'));

        // HACK: We should be calling back to the server to resolve this ref
        // That endpoint doesn't exist yet: https://jira.atlassian.com/browse/STASHDEV-3407
        dialog.updateSelectedRevisionReference(new RevisionReference({
            id : refId,
            displayId : refId.replace(/^refs\/(heads|tags)\//, ''),
            type : /^refs\/tags/.test(refId) ? 'tag' : 'branch'
        }));
    });
});
require('feature/repository/branch-selector-field-internals');
